package com.tzld.longarticle.recommend.server.service.rank;


import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.score.ScoreService;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankService {

    @Autowired
    private ScoreService scoreService;

    public RankResult rank(RankParam param) {

        ScoreResult result = scoreService.score(convertToScoreParam(param));


        Collections.sort(items, (o1, o2) -> Double.compare(o2.getScore(), o1.getScore()));
        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);

        return new RankResult(contents);
    }

    private ScoreParam convertToScoreParam(RankParam param) {
        return null;
    }


    private List<Content> removeDuplicateContent(List<Content> contentList, Set<String> existsContentTitle) {
        List<Content> result = new ArrayList<>();
        for (Content content : contentList) {
            if (existsContentTitle.contains(content.getTitle())
                    || isDuplicateContent(content.getTitle(), existsContentTitle)) {
                continue;
            }
            result.add(content);
            existsContentTitle.add(content.getTitle());
        }
        return result;
    }

    private static final double SIMILARITY_THRESHOLD = 0.8; // 相似度阈值

    private boolean  isDuplicateContent(String title, Set<String> existsContentTitle) {
        boolean result = false;
        for (String existsTitle : existsContentTitle) {
            if (isSimilar(title, existsTitle, SIMILARITY_THRESHOLD)) {
                return true;
            }
        }
        return result;
    }

    private boolean isSimilar(String titleA, String titleB, double threshold) {
        if (titleA.isEmpty() || titleB.isEmpty()) {
            return false;
        }
        Set<Character> setA = new HashSet<>();
        for (char c : titleA.toCharArray()) {
            setA.add(c);
        }
        Set<Character> setB = new HashSet<>();
        for (char c : titleB.toCharArray()) {
            setB.add(c);
        }
        Set<Character> setCross = new HashSet<>(setA);
        setCross.retainAll(setB);
        int minLen = Math.max(Math.min(setA.size(), setB.size()), 1);
        double rate = (double) setCross.size() / minLen;
        return rate >= threshold;
    }

}
